using System.Threading.Tasks;
using UnityEngine;
using UnityEngine.UI;

/// <summary>
/// Type1-Hybrid(仮称) マネージャークラス
/// </summary>
public class Type1HManager : CraneManager
{
    // TODO:
    //  - 2本爪と3本爪の切り替え
    //  - 取り放題モード
    //  - リフトアクション
    //  - 獲得口の仕様刷新
    // memo: [掴む強さ, 上昇中の強さ, 運ぶ途中の強さ]
    [SerializeField] float[] armLPowerConfig = new float[3]; //左アームパワー(%，未確率時)
    [SerializeField] float[] armLPowerConfigSuccess = new float[3]; //左アームパワー(%，確率時)
    [SerializeField] float[] armRPowerConfig = new float[3]; //右アームパワー(%，未確率時)
    [SerializeField] float[] armRPowerConfigSuccess = new float[3]; //右アームパワー(%，確率時)
    public float armLPower = 0f;
    public float armRPower = 0f;
    [SerializeField] float armApertures = 80f; //開口率
    [SerializeField] float[] boxRestrictions = new float[2]; //横・縦の順で移動制限設定
    [SerializeField] float downRestriction = 100f;
    public int soundType = 0; // CLENA:0,1,BIG CLENA:2
    [SerializeField] int catchTime = 2000; //キャッチに要する時間(m秒)
    bool[] isExecuted = new bool[15]; //各craneStatusで1度しか実行しない処理の管理
    public bool buttonPushed = false; //trueならボタンをクリックしているかキーボードを押下している
    [SerializeField] bool player2 = false; //player2の場合true
    [SerializeField] bool button3 = true; //button3の使用可否
    [SerializeField] int[] armSize = new int[2]; // 0:なし，1:S，2:M，3:L
    public Vector2 startPoint; // 開始位置座標定義
    public Vector2 homePoint; // 獲得口座標定義（prizezoneTypeが9のとき使用）
    public int prizezoneType = 9; // 1:左手前，2：左奥，3：右手前，4：右奥，5：左，6：手前，7：右，8：奥，9：特定座標
    TwinArmController armController;
    ArmUnitLifter lifter;
    ArmControllerSupport support;
    CraneBoxSound cbs;
    ArmNail[] nail = new ArmNail[2];
    [SerializeField] TextMesh credit3d;
    [SerializeField] TextMesh[] preset = new TextMesh[4];
    public Animator[] animator = new Animator[4];
    [SerializeField] int armMoveSpeed = 3; // アーム移動速度(1～5)
    [SerializeField] int armDownSpeed = 3; // アーム降下速度(1～5)
    [SerializeField] int armUpSpeed = 3; // アーム上昇速度(1～5)
    //[SerializeField] bool[] prizeHole = new bool[9]; // 落下口(wip)
    //[SerializeField] float homePointZ; // Z座標ホームポジション(%)(wip)

    // レバー操作
    public int operationType = 0; //0:ボタン式，1:レバー式
    public bool leverTilted = false; //trueならレバーがアクティブ
    Lever lever; // コンパネのレバー
    Timer timer;
    [SerializeField] int limitTimeSet = 60; //残り時間を設定
    public Text limitTimedisplayed;
    KeyCode downButtonAlpha;
    KeyCode downButtonNumpad;
    KeyCode catchButtonAlpha;
    KeyCode catchButtonNumpad;

    // エリア別オプション
    //[SerializeField] bool useDownRestrictionByArea; // エリア別下降制限を使う(wip)

    // エラー判定タイマー
    Timer errorTimer;

    // セグ表示
    private const string SEG_PRICE_NOT = "---";
    private const string SEG_CREDIT_NOT = "---";
    private const string SEG_REMAIN_NOT = "-";
    /// <summary>
    /// エラー時のセグ表示
    /// 各プレイヤーのクレジット1価格に表示され、クレジット1プレイ回数にエラーコードが表示される
    /// </summary>
    private const string SEG_PRICE_ERR = "Err";

    /// <summary>
    /// マシンの起動処理
    /// </summary>
    async void Start()
    {
        Transform temp;
        Transform xLimit = transform.Find("Floor").Find("XLimit");
        Transform zLimit = transform.Find("Floor").Find("ZLimit");
        Transform downLimit = transform.Find("Floor").Find("DownLimit");

        craneStatus = -3;
        craneType = 1;

        // 様々なコンポーネントの取得
        //host = transform.root.Find("CP").GetComponent<MachineHost>();
        canvas = transform.Find("Canvas").gameObject;
        creditSystem = transform.Find("CreditSystem").GetComponent<CreditSystem>();
        //sp = transform.Find("SE").GetComponent<SEPlayer>();
        getPoint = transform.Find("Floor").Find("GetPoint").GetComponent<GetPoint>();
        temp = transform.Find("CraneUnit").transform;
        lever = transform.Find("Canvas").Find("ControlGroup").Find("Lever 1").GetComponent<Lever>();

        errorTimer = transform.Find("ErrorTimer").GetComponent<Timer>();

        // クレジット情報登録
        creditSystem.rateSet[0, 0] = priceSet[0];
        creditSystem.rateSet[1, 0] = priceSet[1];
        creditSystem.rateSet[0, 1] = timesSet[0];
        creditSystem.rateSet[1, 1] = timesSet[1];
        if (isHibernate)
        {
            creditSystem.SetHibernate();
            this.SetSegDisplay(9);
        }
        else
        {
            this.SetSegDisplay(0);
        }

        // ロープとアームコントローラに関する処理
        lifter = temp.Find("CraneBox").Find("Tube").Find("TubePoint").GetComponent<ArmUnitLifter>();
        armController = temp.Find("ArmUnit").GetComponent<TwinArmController>();
        support = temp.Find("ArmUnit").Find("Main").GetComponent<ArmControllerSupport>();

        for (int i = 0; i < 2; i++)
        {
            string a = "Arm" + (i + 1).ToString();
            GameObject arm;
            switch (armSize[i])
            {
                case 0:
                case 1:
                    a += "S";
                    break;
                case 2:
                    a += "M";
                    break;
                case 3:
                    a += "L";
                    break;
            }
            arm = temp.Find("ArmUnit").Find(a).gameObject;
            nail[i] = arm.transform.Find("Nail").GetComponent<ArmNail>();
            if (armSize[i] != 0) arm.SetActive(true);
            armController.SetArm(i, armSize[i]);
        }

        await Task.Delay(500);

        // CraneBoxに関する処理
        craneBox = temp.Find("CraneBox").GetComponent<CraneBox>();
        cbs = temp.Find("CraneBox").GetComponent<CraneBoxSound>();

        // ロープにマネージャー情報をセット
        creditSystem.SetSEPlayer(sp);
        getPoint.SetManager(this);
        lifter.Up();
        creditSystem.SetCreditSound(0);
        creditSystem.SetSEPlayer(sp);
        support.SetManager(this);
        support.SetLifter(lifter);
        support.pushTime = 300; // 押し込みパワーの調整
        getSoundNum = 5;
        for (int i = 0; i < 2; i++)
        {
            nail[i].SetManager(this);
            nail[i].SetLifter(lifter);
        }

        for (int i = 0; i < 14; i++)
            isExecuted[i] = false;

        isExecuted[14] = true; //初回処理用（レバーを倒しっぱなしでプレイし始めるときの対策）

        // UIの表示を変える
        if (operationType == 0)
        {
            transform.Find("Canvas").Find("ControlGroup").Find("Lever Hole").gameObject.SetActive(false);
            transform.Find("Canvas").Find("ControlGroup").Find("Lever 1").gameObject.SetActive(false);
            transform.Find("Canvas").Find("ControlGroup").Find("Lever 2").gameObject.SetActive(false);

            if (!button3)
            {
                transform.Find("Canvas").Find("ControlGroup").Find("Button 3").gameObject.SetActive(false);
                transform.Find("Floor").Find("Button3").gameObject.SetActive(false);
            }
            else transform.Find("Floor").Find("Button3Disabled").gameObject.SetActive(false);
        }
        else if (operationType == 1)
        {
            //transform.Find("Canvas").Find("ControlGroup").Find("Button 1").gameObject.SetActive(false);
            //transform.Find("Canvas").Find("ControlGroup").Find("Button 2").gameObject.SetActive(false);
            //transform.Find("Canvas").Find("ControlGroup").Find("Button 3").gameObject.SetActive(false);
            transform.Find("Canvas").Find("ControlGroup").Find("Button 3 Disabled").gameObject.SetActive(false);

            timer = transform.Find("Timer").GetComponent<Timer>();
            timer.limitTime = limitTimeSet;
        }
        transform.Find("Canvas").Find("ControlGroup").Find("Button 1").gameObject.SetActive(operationType == 0);
        transform.Find("Canvas").Find("ControlGroup").Find("Button 2").gameObject.SetActive(operationType == 0);
        transform.Find("Canvas").Find("ControlGroup").Find("Button 3").gameObject.SetActive(operationType == 0);

        transform.Find("Canvas").Find("ControlGroup").Find("Lever Hole").gameObject.SetActive(operationType == 1);
        transform.Find("Canvas").Find("ControlGroup").Find("Lever 1").gameObject.SetActive(operationType == 1);
        transform.Find("Canvas").Find("ControlGroup").Find("Lever 2").gameObject.SetActive(operationType == 1);

        // 降下ボタンと下降停止ボタンの割当
        if (operationType == 0)
        {
            downButtonAlpha = !player2 ? KeyCode.Alpha2 : KeyCode.Alpha8;
            downButtonNumpad = !player2 ? KeyCode.Keypad2 : KeyCode.Keypad8;
            catchButtonAlpha = !player2 ? KeyCode.Alpha3 : KeyCode.Alpha9;
            catchButtonNumpad = !player2 ? KeyCode.Keypad3 : KeyCode.Keypad9;
        }
        else if (operationType == 1)
        {
            downButtonAlpha = !player2 ? KeyCode.Alpha2 : KeyCode.Alpha8;
            downButtonNumpad = !player2 ? KeyCode.Keypad2 : KeyCode.Keypad8;
            catchButtonAlpha = !player2 ? KeyCode.Alpha2 : KeyCode.Alpha8;
            catchButtonNumpad = !player2 ? KeyCode.Keypad2 : KeyCode.Keypad8;
        }

        // イニシャル移動とinsertFlagを後に実行
        while (!lifter.UpFinished())
        {
            await Task.Delay(100);
        }
        armController.SetLimit(armApertures);

        // TODO: アーム移動速度設定
        // 開始位置への移動前に設定すること
        this.SetArmMoveSpeed(armMoveSpeed);
        this.SetArmLiftSpeed(armUpSpeed, armDownSpeed);

        // ホームポジション、開放座標を算出する
        startPoint = GetCorrectedHomePosition(player2, startPoint, armSize);
        homePoint = GetCorrectedHomePosition(player2, homePoint, armSize);

        // TODO: Z座標のホームポジションを算出する
        // CraneBox側ではなくArmUnitLifter側

        // 移動限界座標を算出
        if (!player2)
        {
            // 1P
            if (boxRestrictions[0] < 100) xLimit.localPosition = new Vector3(-0.55f + 0.005025f * boxRestrictions[0], xLimit.localPosition.y, xLimit.localPosition.z);
        }
        else
        {
            // 2P
            if (boxRestrictions[0] < 100) xLimit.localPosition = new Vector3(0.55f - 0.005025f * boxRestrictions[0], xLimit.localPosition.y, xLimit.localPosition.z);
        }
        if (boxRestrictions[1] < 100) zLimit.localPosition = new Vector3(zLimit.localPosition.x, zLimit.localPosition.y, -0.2f + 0.00615f * boxRestrictions[1]);
        if (downRestriction < 100) downLimit.localPosition = new Vector3(downLimit.localPosition.x, 1.39f - 0.0068f * downRestriction, downLimit.localPosition.z);

        // 開始位置に移動させる
        craneBox.goPoint = startPoint;

        // TODO: この時点でホームポジション,開放位置が窓ガラスを完全に超えるようならエラーを出してもいいかも
        // (結局ホームポジションに到達できなくなって詰む)

        host.manualCode = 1;
        craneStatus = -2;
        cbs.MoveSound(true);
    }

    async void Update()
    {
        if (useUI && host.playable && !canvas.activeSelf) canvas.SetActive(true);
        else if (!host.playable && canvas.activeSelf) canvas.SetActive(false);
        if (!player2 && (Input.GetKeyDown(KeyCode.Keypad0) || Input.GetKeyDown(KeyCode.Alpha0))) InsertCoin();
        else if (player2 && (Input.GetKeyDown(KeyCode.KeypadPeriod) || Input.GetKeyDown(KeyCode.Minus))) InsertCoin();

        if (craneStatus == -2 && ((craneBox.CheckPos(1) && !player2) || (craneBox.CheckPos(3) && player2)))
        {
            cbs.MoveSound(false);
            craneStatus = -1;
            craneBox.goPositionFlag = true;
            cbs.MoveSound(true);
        }
        if (craneStatus == -1)
        {
            // TODO: X,Y移動完了してからZ移動フェーズ
            if (craneBox.CheckPos(9))
            {
                cbs.MoveSound(false);
                craneStatus = 0;
            }
        }

        if (craneStatus > 0)
        {
            if (Input.GetKey(KeyCode.M) && Input.GetKey(KeyCode.Y) && Input.GetKey(KeyCode.C) && !probability) probability = true; // テスト用隠しコマンド
            if (operationType == 0)
            {
                if (craneStatus == 1)
                {
                    //コイン投入有効化;
                    //右移動ボタン有効化;
                    DetectKey(craneStatus);
                }

                if (craneStatus == 2)
                { //右移動中
                  //コイン投入無効化;
                    if (!isExecuted[craneStatus])
                    {
                        isExecuted[craneStatus] = true;
                        cbs.MoveSound(true);
                    }
                    DetectKey(craneStatus);
                    //クレーン右移動;
                    switch (soundType)
                    {
                        case 0:
                        case 1:
                            sp.Play(1, 2);
                            break;
                        case 2:
                            sp.Play(7, 2);
                            break;
                    }

                    if (!player2 && craneBox.CheckPos(7))
                    {
                        buttonPushed = false;
                        craneStatus = 3;
                    }
                    if (player2 && craneBox.CheckPos(5))
                    {
                        buttonPushed = false;
                        craneStatus = 3;
                    }
                    //右移動効果音ループ再生;
                }

                if (craneStatus == 3)
                {
                    if (!isExecuted[craneStatus])
                    {
                        isExecuted[craneStatus] = true;
                        cbs.MoveSound(false);
                    }
                    DetectKey(craneStatus);
                    switch (soundType)
                    {
                        case 0:
                        case 1:
                            sp.Stop(1);
                            break;
                        case 2:
                            sp.Stop(7);
                            break;
                    } //右移動効果音ループ再生停止;
                      //奥移動ボタン有効化;
                }

                if (craneStatus == 4)
                { //奥移動中
                  //クレーン奥移動;
                    if (!isExecuted[craneStatus])
                    {
                        isExecuted[craneStatus] = true;
                        cbs.MoveSound(true);
                    }
                    DetectKey(craneStatus);
                    switch (soundType)
                    {
                        case 0:
                        case 1:
                            sp.Play(1, 2);
                            break;
                        case 2:
                            sp.Play(7, 2);
                            break;
                    }
                    if (craneBox.CheckPos(8))
                    {
                        buttonPushed = false;
                        craneStatus = 5;
                    }
                    //奥移動効果音ループ再生;
                }
            }
            else if (operationType == 1)
            {
                if (craneStatus == 1)
                {
                    if (!isExecuted[craneStatus])
                    {
                        isExecuted[craneStatus] = true;
                    }

                    if (!player2)
                    {
                        if (isExecuted[14] && ((Input.GetKey(KeyCode.H) || Input.GetKey(KeyCode.F) || Input.GetKey(KeyCode.T) || Input.GetKey(KeyCode.G)
                        || lever.rightFlag || lever.leftFlag || lever.backFlag || lever.forwardFlag) && !leverTilted && host.playable)) // 初回用の処理
                        {
                            leverTilted = true;
                            isExecuted[14] = false;
                            sp.Play((int)TYPE1_SE.CLENA_FLEX_MOVE);
                            cbs.MoveSound(true);
                        }

                        if ((Input.GetKeyDown(KeyCode.H) || Input.GetKeyDown(KeyCode.F) || Input.GetKeyDown(KeyCode.T) || Input.GetKeyDown(KeyCode.G)
                        || lever.rightFlag || lever.leftFlag || lever.backFlag || lever.forwardFlag) && !leverTilted && host.playable)
                        {
                            leverTilted = true;
                            sp.Play((int)TYPE1_SE.CLENA_FLEX_MOVE);
                            cbs.MoveSound(true);
                        }
                        if (((!Input.GetKey(KeyCode.H) && !Input.GetKey(KeyCode.F) && !Input.GetKey(KeyCode.T) && !Input.GetKey(KeyCode.G)
                        && !lever.rightFlag && !lever.leftFlag && !lever.backFlag && !lever.forwardFlag) && leverTilted && host.playable) || (leverTilted && !host.playable))
                        {
                            leverTilted = false;
                            sp.Stop((int)TYPE1_SE.CLENA_FLEX_MOVE);
                            cbs.MoveSound(false);
                        }
                    }
                    else
                    {
                        if (isExecuted[14] && ((Input.GetKey(KeyCode.L) || Input.GetKey(KeyCode.J) || Input.GetKey(KeyCode.I) || Input.GetKey(KeyCode.K)
                        || lever.rightFlag || lever.leftFlag || lever.backFlag || lever.forwardFlag) && !leverTilted && host.playable)) // 初回用の処理
                        {
                            leverTilted = true;
                            isExecuted[14] = false;
                            sp.Play((int)TYPE1_SE.CLENA_FLEX_MOVE);
                            cbs.MoveSound(true);
                        }

                        if ((Input.GetKeyDown(KeyCode.L) || Input.GetKeyDown(KeyCode.J) || Input.GetKeyDown(KeyCode.I) || Input.GetKeyDown(KeyCode.K)
                        || lever.rightFlag || lever.leftFlag || lever.backFlag || lever.forwardFlag) && !leverTilted && host.playable)
                        {
                            leverTilted = true;
                            sp.Play((int)TYPE1_SE.CLENA_FLEX_MOVE);
                            cbs.MoveSound(true);
                        }
                        if (((!Input.GetKey(KeyCode.L) && !Input.GetKey(KeyCode.J) && !Input.GetKey(KeyCode.I) && !Input.GetKey(KeyCode.K)
                        && !lever.rightFlag && !lever.leftFlag && !lever.backFlag && !lever.forwardFlag) && leverTilted && host.playable) || (leverTilted && !host.playable))
                        {
                            leverTilted = false;
                            sp.Stop((int)TYPE1_SE.CLENA_FLEX_MOVE);
                            cbs.MoveSound(false);
                        }
                    }
                }
                if (craneStatus == 3)
                {
                    if (!isExecuted[craneStatus])
                    {
                        isExecuted[craneStatus] = true;
                        timer.StartTimer();
                        creditSystem.segUpdateFlag = false;
                    }

                    if (!player2)
                    {
                        if (isExecuted[14] && ((Input.GetKey(KeyCode.H) || Input.GetKey(KeyCode.F) || Input.GetKey(KeyCode.T) || Input.GetKey(KeyCode.G)
                        || lever.rightFlag || lever.leftFlag || lever.backFlag || lever.forwardFlag) && !leverTilted && host.playable)) // 初回用の処理
                        {
                            leverTilted = true;
                            isExecuted[14] = false;
                            sp.Play((int)TYPE1_SE.CLENA_FLEX_MOVE);
                            cbs.MoveSound(true);
                        }

                        if ((Input.GetKeyDown(KeyCode.H) || Input.GetKeyDown(KeyCode.F) || Input.GetKeyDown(KeyCode.T) || Input.GetKeyDown(KeyCode.G)
                        || lever.rightFlag || lever.leftFlag || lever.backFlag || lever.forwardFlag) && !leverTilted && host.playable)
                        {
                            leverTilted = true;
                            sp.Play((int)TYPE1_SE.CLENA_FLEX_MOVE);
                            cbs.MoveSound(true);
                        }
                        if (((!Input.GetKey(KeyCode.H) && !Input.GetKey(KeyCode.F) && !Input.GetKey(KeyCode.T) && !Input.GetKey(KeyCode.G)
                        && !lever.rightFlag && !lever.leftFlag && !lever.backFlag && !lever.forwardFlag) && leverTilted && host.playable) || (leverTilted && !host.playable))
                        {
                            leverTilted = false;
                            sp.Stop((int)TYPE1_SE.CLENA_FLEX_MOVE);
                            cbs.MoveSound(false);
                        }
                    }
                    else
                    {
                        if (isExecuted[14] && ((Input.GetKey(KeyCode.L) || Input.GetKey(KeyCode.J) || Input.GetKey(KeyCode.I) || Input.GetKey(KeyCode.K)
                        || lever.rightFlag || lever.leftFlag || lever.backFlag || lever.forwardFlag) && !leverTilted && host.playable)) // 初回用の処理
                        {
                            leverTilted = true;
                            isExecuted[14] = false;
                            sp.Play((int)TYPE1_SE.CLENA_FLEX_MOVE);
                            cbs.MoveSound(true);
                        }

                        if ((Input.GetKeyDown(KeyCode.L) || Input.GetKeyDown(KeyCode.J) || Input.GetKeyDown(KeyCode.I) || Input.GetKeyDown(KeyCode.K)
                        || lever.rightFlag || lever.leftFlag || lever.backFlag || lever.forwardFlag) && !leverTilted && host.playable)
                        {
                            leverTilted = true;
                            sp.Play((int)TYPE1_SE.CLENA_FLEX_MOVE);
                            cbs.MoveSound(true);
                        }
                        if (((!Input.GetKey(KeyCode.L) && !Input.GetKey(KeyCode.J) && !Input.GetKey(KeyCode.I) && !Input.GetKey(KeyCode.K)
                        && !lever.rightFlag && !lever.leftFlag && !lever.backFlag && !lever.forwardFlag) && leverTilted && host.playable) || (leverTilted && !host.playable))
                        {
                            leverTilted = false;
                            sp.Stop((int)TYPE1_SE.CLENA_FLEX_MOVE);
                            cbs.MoveSound(false);
                        }

                    }
                    if (!leverTilted) DetectKey(5);
                    if (isExecuted[craneStatus] && timer.limitTimeNow <= 0)
                    {
                        // 時間切れになったら強制的に下降へ
                        craneStatus = (int)TYPE1_STATUS.STATUS_Y_MOVE_END;
                    }
                }
            }

            if (craneStatus == 5)
            {
                switch (soundType)
                {
                    case 0:
                    case 1:
                        sp.Stop(1);
                        break;
                    case 2:
                        sp.Stop(7);
                        break;
                } //奥移動効果音ループ再生停止;
                if (!isExecuted[craneStatus])
                {
                    isExecuted[craneStatus] = true;
                    if (operationType == 0)
                        cbs.MoveSound(false); // ボタン操作の場合はステッピングモーター音を停止する
                    else if (operationType == 1)
                    {
                        timer.CancelTimer(); // レバー操作時のタイマー終了
                        creditSystem.segUpdateFlag = true;
                        this.SetSegDisplay(0);
                    }
                    armController.Open();
                    await Task.Delay(1000);
                    if (craneStatus == 5) craneStatus = 6;
                }
                //アーム開く音再生;
                //アーム開く;
            }

            if (craneStatus == 6)
            {
                if (!isExecuted[craneStatus])
                {
                    isExecuted[craneStatus] = true;
                    if (craneStatus == 6) lifter.Down(); //awaitによる時差実行を防止
                }
                DetectKey(craneStatus);
                switch (soundType)
                {
                    case 0:
                    case 1:
                        sp.Play(2, 2);
                        break;
                    case 2:
                        sp.Play(8, 2);
                        break;
                }
                if (lifter.DownFinished() && craneStatus == 6) craneStatus = 7;
                //アーム下降音再生
                //アーム下降;
            }

            if (craneStatus == 7)
            {
                switch (soundType)
                {
                    case 0:
                    case 1:
                        sp.Stop(2);
                        break;
                    case 2:
                        sp.Stop(8);
                        break;
                } //アーム下降音再生停止;
                await Task.Delay(1000);
                if (craneStatus == 7)
                {
                    switch (soundType)
                    {
                        case 0:
                        case 1:
                            sp.Play(3, 2);
                            break;
                        case 2:
                            sp.Play(9, 2);
                            break;
                    }
                } //アーム掴む音再生;
                if (!isExecuted[craneStatus])
                {
                    isExecuted[craneStatus] = true;
                    if (craneStatus == 7)
                    {
                        // アームパワー再設定
                        // 閉じるときのアームパワーは大きい方を採用．最低値は30f
                        armLPower = probability ? armLPowerConfigSuccess[0] : armLPowerConfig[0];
                        armRPower = probability ? armRPowerConfigSuccess[0] : armRPowerConfig[0];

                        if (armLPower >= 30 || armRPower >= 30)
                        {
                            // アームパワーが30以上ならアームパワーの大きい方を基準にして閉じる
                            armController.Close(armLPower >= armRPower ? armLPower : armRPower);
                        }
                        else
                        {
                            // 最低値に達していないので30fで閉じる
                            armController.Close(30f);
                        }

                        armController.SetMotorPower(armLPower, 0);
                        armController.SetMotorPower(armRPower, 1);
                    }
                    await Task.Delay(catchTime);
                    if (craneStatus == 7) craneStatus = 8; //awaitによる時差実行を防止
                }
                //アーム掴む;
            }

            if (craneStatus == 8)
            {
                switch (soundType)
                {
                    case 0:
                    case 1:
                        sp.Stop(3);
                        sp.Play(4, 2);
                        break;
                    case 2:
                        sp.Stop(9);
                        sp.Play(10, 2);
                        break;
                } //アーム上昇音再生;
                if (!isExecuted[craneStatus])
                {
                    isExecuted[craneStatus] = true;
                    errorTimer.StartTimer();
                    lifter.Up();
                    await Task.Delay(1000);
                    if (craneStatus < 11)
                    {
                        armController.SetMotorPower(probability ? armLPowerConfigSuccess[0] : armLPowerConfig[0], 0);
                        armController.SetMotorPower(probability ? armRPowerConfigSuccess[0] : armRPowerConfig[0], 1);
                    }
                }
                if (lifter.UpFinished() && craneStatus == 8) craneStatus = 9;
                //アーム上昇;
            }

            if (craneStatus == 9)
            {
                //アーム上昇停止音再生;
                //アーム上昇停止;
                if (!isExecuted[craneStatus])
                {
                    isExecuted[craneStatus] = true;
                    switch (soundType)
                    {
                        case 0:
                        case 1:
                            sp.Stop(4);
                            break;
                        case 2:
                            sp.Stop(10);
                            break;
                    }
                    errorTimer.CancelTimer();
                    await Task.Delay(500);
                    if (craneStatus == 9)
                    {
                        errorTimer.StartTimer();
                        // パワー調整
                        armLPower = probability ? armLPowerConfigSuccess[1] : armLPowerConfig[1];
                        armRPower = probability ? armRPowerConfigSuccess[1] : armRPowerConfig[1];

                        armController.SetMotorPower(armLPower, 0);
                        armController.SetMotorPower(armRPower, 1);

                        if (prizezoneType == 9)
                        {
                            craneBox.goPoint = homePoint;
                            craneBox.goPositionFlag = true;
                        }
                        cbs.MoveSound(true);
                        craneStatus = 10;
                    }
                }
            }

            if (craneStatus == 10)
            {
                if (craneBox.CheckPos(prizezoneType))
                {
                    craneStatus = 11;
                }
                //アーム獲得口ポジション移動音再生;
                //アーム獲得口ポジションへ;
            }

            if (craneStatus == 11)
            {
                if (!isExecuted[craneStatus])
                {
                    isExecuted[craneStatus] = true;
                    errorTimer.CancelTimer();
                    cbs.MoveSound(false);
                    //armController.SetLimit(100f); // アーム開口度を100に
                    armController.Open();
                    await Task.Delay(2000);
                    if (craneStatus == 11) craneStatus = 12;
                }
                //アーム開く音再生;
                //アーム開く;
                //1秒待機;
            }

            if (craneStatus == 12)
            {
                if (!isExecuted[craneStatus])
                {
                    isExecuted[craneStatus] = true;
                    armController.Close(50f);
                    await Task.Delay(1000);
                    if (craneStatus == 12)
                    {
                        cbs.MoveSound(true);
                        craneStatus = 13;
                    }
                }
                //アーム閉じる音再生;
                //アーム閉じる;
                //1秒待機;
            }

            if (craneStatus == 13)
            {
                if ((craneBox.CheckPos(1) && !player2) || (craneBox.CheckPos(3) && player2))
                {
                    if (!isExecuted[craneStatus])
                    {
                        isExecuted[craneStatus] = true;
                        errorTimer.limitTime = 10;
                        cbs.MoveSound(false);
                    }
                    await Task.Delay(1000);
                    if (craneStatus == 13) craneStatus = 14;
                }
                //アーム初期位置帰還
            }
            if (craneStatus == 14)
            {
                if (!isExecuted[craneStatus])
                {
                    isExecuted[craneStatus] = true;
                    errorTimer.CancelTimer();
                    for (int i = 0; i < 14; i++)
                        isExecuted[i] = false;
                    armController.SetLimit(armApertures); //アーム開口度リセット

                    craneBox.goPoint = startPoint;
                    craneBox.goPositionFlag = true;
                    cbs.MoveSound(true);
                }
                if (isExecuted[craneStatus])
                {
                    if (craneBox.CheckPos(9))
                    {
                        cbs.MoveSound(false);
                        if (creditSystem.creditDisplayed > 0)
                            craneStatus = 1;
                        else
                            craneStatus = 0;
                    }
                }
                //アームスタート位置へ
            }

            if (!creditSystem.segUpdateFlag) // Timer表示用
            {
                this.SetSegDisplay(1);
            }
            if (errorTimer.limitTimeNow == 0) // Error
            {
                if (!isExecuted[0])
                {
                    isExecuted[0] = true;
                    Debug.Log("Type1H Error! CraneStatus: " + craneStatus);
                    // アームの動きを停止、休止に移す
                    cbs.MoveSound(false);
                    lifter.UpForceStop();
                    lifter.DownForceStop();
                    craneBox.MoveForceStop();
                    isHibernate = true;

                    // セグメント表示
                    this.SetSegDisplay(8);

                    // 効果音停止
                    switch (soundType)
                    {
                        case 0:
                        case 1:
                            sp.Stop(1);
                            sp.Stop(2);
                            sp.Stop(3);
                            sp.Stop(4);
                            break;

                        case 2:
                            sp.Stop(7);
                            sp.Stop(8);
                            sp.Stop(9);
                            sp.Stop(10);
                            break;
                    };

                    // エラー音再生
                    sp.Play((int)TYPE1_SE.CLENA_FLEX_ERROR);
                }
                craneStatus = 99;
            }
        }
    }

    void FixedUpdate()
    {
        if (craneStatus != 0)
        {
            if (craneStatus == -2 || craneStatus == 13)
            {
                if (!player2) craneBox.Left();
                else craneBox.Right();
                craneBox.Forward();
            }
            else if (craneStatus == 8)
            {
                // アームパワーを徐々に減らす
                if (probability)
                {
                    if (armLPower > armLPowerConfigSuccess[1])
                    {
                        armLPower -= 0.15f;
                    }
                    if (armRPower > armRPowerConfigSuccess[1])
                    {
                        armRPower -= 0.15f;
                    }
                }
                else
                {
                    if (armLPower > armLPowerConfig[1])
                    {
                        armLPower -= 0.15f;
                    }
                    if (armRPower > armRPowerConfig[1])
                    {
                        armRPower -= 0.15f;
                    }
                }
                armController.SetMotorPower(armLPower, 0);
                armController.SetMotorPower(armRPower, 1);
            }
            else if (craneStatus == 10 || craneStatus == 15)
            {
                switch (prizezoneType) // 1:左手前，2：左奥，3：右手前，4：右奥，5：左，6：手前，7：右，8：奥，9：特定座標
                {
                    case 1:
                        craneBox.Left();
                        craneBox.Forward();
                        break;
                    case 2:
                        craneBox.Left();
                        craneBox.Back();
                        break;
                    case 3:
                        craneBox.Right();
                        craneBox.Forward();
                        break;
                    case 4:
                        craneBox.Right();
                        craneBox.Back();
                        break;
                    case 5:
                        craneBox.Left();
                        break;
                    case 6:
                        craneBox.Forward();
                        break;
                    case 7:
                        craneBox.Right();
                        break;
                    case 8:
                        craneBox.Back();
                        break;
                }
            }
            else if (craneStatus == 2)
            {
                if (!player2) craneBox.Right();
                else craneBox.Left();
            }
            else if (craneStatus == 4) craneBox.Back();
            else if (craneStatus == 1 || craneStatus == 3)
            {
                DetectLever();
            }
        }
    }

    public override void GetPrize()
    {
        for (int i = 0; i < 4; i++)
            animator[i].SetTrigger("GetPrize");
        base.GetPrize();
    }

    /// <summary>
    /// キー入力
    /// </summary>
    protected override void DetectKey(int num)
    {
        if (host.playable)
        {
            int credit = 0;
            switch (num)
            {
                case 1:
                    if ((Input.GetKeyDown(KeyCode.Keypad1) || Input.GetKeyDown(KeyCode.Alpha1)) && !buttonPushed && !player2)
                    {
                        buttonPushed = true;
                        if (craneStatus == 1)
                        {
                            creditSystem.ResetPayment();
                            credit = creditSystem.PlayStart();
                            if (credit < 10) credit3d.text = credit.ToString();
                            else credit3d.text = "9.";
                            isExecuted[14] = false;
                            // 確率判定
                            probability = creditSystem.ProbabilityCheck();
                            Debug.Log("Probability:" + probability);
                        }
                        craneStatus = 2;
                    }
                    if ((Input.GetKeyDown(KeyCode.Keypad7) || Input.GetKeyDown(KeyCode.Alpha7)) && !buttonPushed && player2)
                    {
                        buttonPushed = true;
                        if (craneStatus == 1)
                        {
                            creditSystem.ResetPayment();
                            credit = creditSystem.PlayStart();
                            if (credit < 10) credit3d.text = credit.ToString();
                            else credit3d.text = "9.";
                            isExecuted[14] = false;
                            probability = creditSystem.ProbabilityCheck();
                            Debug.Log("Probability:" + probability);
                        }
                        craneStatus = 2;
                    }
                    break;
                //投入を無効化
                case 2:
                    if ((Input.GetKeyUp(KeyCode.Keypad1) || Input.GetKeyUp(KeyCode.Alpha1)) && buttonPushed && !player2)
                    {
                        craneStatus = 3;
                        buttonPushed = false;
                    }
                    if ((Input.GetKeyUp(KeyCode.Keypad7) || Input.GetKeyUp(KeyCode.Alpha7)) && buttonPushed && player2)
                    {
                        craneStatus = 3;
                        buttonPushed = false;
                    }
                    break;
                case 3:
                    if ((Input.GetKeyDown(KeyCode.Keypad2) || Input.GetKeyDown(KeyCode.Alpha2)) && !buttonPushed && !player2)
                    {
                        buttonPushed = true;
                        craneStatus = 4;
                    }
                    if ((Input.GetKeyDown(KeyCode.Keypad8) || Input.GetKeyDown(KeyCode.Alpha8)) && !buttonPushed && player2)
                    {
                        buttonPushed = true;
                        craneStatus = 4;
                    }
                    break;
                case 4:
                    if ((Input.GetKeyUp(KeyCode.Keypad2) || Input.GetKeyUp(KeyCode.Alpha2)) && buttonPushed && !player2)
                    {
                        craneStatus = 5;
                        buttonPushed = false;
                    }
                    if ((Input.GetKeyUp(KeyCode.Keypad8) || Input.GetKeyUp(KeyCode.Alpha8)) && buttonPushed && player2)
                    {
                        craneStatus = 5;
                        buttonPushed = false;
                    }
                    break;
                case 5:
                    // レバー用
                    if ((Input.GetKeyDown(downButtonNumpad) || Input.GetKeyDown(downButtonAlpha)) && !buttonPushed)
                    {
                        craneStatus = 5;
                        buttonPushed = false;
                    }
                    break;
                case 6:
                    // 下降停止ボタン
                    if ((Input.GetKeyDown(catchButtonNumpad) || Input.GetKeyDown(catchButtonAlpha)) && button3)
                    {
                        lifter.DownForceStop();
                        craneStatus = 7;
                    }
                    break;
            }
        }
    }

    public void DetectLever() // キーボード，UI共通のレバー処理
    {
        if (host.playable)
        {
            int credit = 0;
            if (!player2)
            {
                if (Input.GetKey(KeyCode.H) || lever.rightFlag)
                    craneBox.Right();
                if (Input.GetKey(KeyCode.F) || lever.leftFlag)
                    craneBox.Left();
                if (Input.GetKey(KeyCode.T) || lever.backFlag)
                    craneBox.Back();
                if (Input.GetKey(KeyCode.G) || lever.forwardFlag)
                    craneBox.Forward();

                if (Input.GetKey(KeyCode.H) || Input.GetKey(KeyCode.F) || Input.GetKey(KeyCode.T) || Input.GetKey(KeyCode.G)
                || lever.rightFlag || lever.leftFlag || lever.backFlag || lever.forwardFlag)
                {
                    if (craneStatus == 1)
                    {
                        creditSystem.ResetPayment();
                        credit = creditSystem.PlayStart();
                        if (credit < 10) credit3d.text = credit.ToString();
                        else credit3d.text = "9.";
                        isExecuted[14] = false;
                        // 確率判定
                        probability = creditSystem.ProbabilityCheck();
                        Debug.Log("Probability:" + probability);
                        craneStatus = 3;
                    }
                }
            }
            else //2Pレバー
            {
                if (Input.GetKey(KeyCode.L) || lever.rightFlag)
                    craneBox.Right();
                if (Input.GetKey(KeyCode.J) || lever.leftFlag)
                    craneBox.Left();
                if (Input.GetKey(KeyCode.I) || lever.backFlag)
                    craneBox.Back();
                if (Input.GetKey(KeyCode.K) || lever.forwardFlag)
                    craneBox.Forward();

                if (Input.GetKey(KeyCode.L) || Input.GetKey(KeyCode.J) || Input.GetKey(KeyCode.I) || Input.GetKey(KeyCode.K)
                || lever.rightFlag || lever.leftFlag || lever.backFlag || lever.forwardFlag)
                {
                    if (craneStatus == 1)
                    {
                        creditSystem.ResetPayment();
                        credit = creditSystem.PlayStart();
                        if (credit < 10) credit3d.text = credit.ToString();
                        else credit3d.text = "9.";
                        isExecuted[14] = false;
                        // 確率判定
                        probability = creditSystem.ProbabilityCheck();
                        Debug.Log("Probability:" + probability);
                        craneStatus = 3;
                    }
                }
            }
        }
    }

    /// <summary>
    /// ボタン押下
    /// </summary>
    /// <param name="num"></param>
    public override void ButtonDown(int num)
    {
        int credit = 0;
        switch (num)
        {
            case 1:
                if (craneStatus == 1 && !buttonPushed)
                {
                    buttonPushed = true;
                    craneStatus = 2;
                    creditSystem.ResetPayment();
                    credit = creditSystem.PlayStart();
                    if (credit < 10) credit3d.text = credit.ToString();
                    else credit3d.text = "9.";
                    isExecuted[14] = false;
                    probability = creditSystem.ProbabilityCheck();
                    Debug.Log("Probability:" + probability);
                }
                break;
            case 2:
                if ((craneStatus == 3 && !buttonPushed) || (craneStatus == 4 && buttonPushed))
                {
                    buttonPushed = true;
                    craneStatus = 4;
                }
                break;
            case 3:
                if (operationType == 0 && craneStatus == 6)
                {
                    // ボタン操作
                    lifter.DownForceStop();
                    craneStatus = 7;
                }
                else if (operationType == 1)
                {
                    // レバー操作の場合は1つのボタンで下降/下降停止を行う
                    if (craneStatus == 3)
                    {
                        buttonPushed = true;
                        craneStatus = 5;
                    }
                    else if (craneStatus == 6)
                    {
                        lifter.DownForceStop();
                        craneStatus = 7;
                    }
                }
                break;
            case 4: // player2 case 1:
                if (craneStatus == 1 && !buttonPushed)
                {
                    buttonPushed = true;
                    craneStatus = 2;
                    creditSystem.ResetPayment();
                    credit = creditSystem.PlayStart();
                    if (credit < 10) credit3d.text = credit.ToString();
                    else credit3d.text = "9.";
                    isExecuted[14] = false;
                    probability = creditSystem.ProbabilityCheck();
                    Debug.Log("Probability:" + probability);
                }
                break;
        }
    }

    /// <summary>
    /// ボタンが離された
    /// </summary>
    /// <param name="num"></param>
    public override void ButtonUp(int num)
    {
        switch (num)
        {
            case 1:
                if (craneStatus == 2 && buttonPushed)
                {
                    craneStatus = 3;
                    buttonPushed = false;
                }
                break;
            case 2:
                if (craneStatus == 4 && buttonPushed)
                {
                    craneStatus = 5;
                    buttonPushed = false;
                }
                break;
            case 4: // player2 case 1:
                if (craneStatus == 2 && buttonPushed)
                {
                    craneStatus = 3;
                    buttonPushed = false;
                }
                break;
        }
    }

    /// <summary>
    /// クレジット投入
    /// </summary>
    public override void InsertCoin()
    {
        if (!isHibernate && host.playable && craneStatus >= 0)
        {
            int credit = creditSystem.Pay(100);
            if (credit < 10) credit3d.text = credit.ToString();
            else credit3d.text = "9.";
            if (credit > 0 && craneStatus == 0) craneStatus = 1;
        }
    }

    /// <summary>
    /// クレジット投入(オートプレイ時)
    /// </summary>
    public override void InsertCoinAuto()
    {
        if (!isHibernate && craneStatus >= 0)
        {
            int credit = creditSystem.Pay(100);
            if (credit < 10) credit3d.text = credit.ToString();
            else credit3d.text = "9.";
            if (credit > 0 && craneStatus == 0) craneStatus = 1;
        }
    }

    /// <summary>
    /// アームサイズを考慮した開始位置座標を計算して返す
    /// </summary>
    /// <remarks>
    /// 指定した座標がアームサイズを考慮した座標よりも外側だった場合のみ、補正した値を返します。
    /// 現状アームユニットの向きが正面前提なので、90度回転を行う場合は使わないことを推奨します。
    /// </remarks>
    /// <param name="_player2">2P側であるか</param>
    /// <param name="_position">補正なしの座標</param>
    /// <param name="_armSize">アームサイズ</param>
    /// <returns>座標</returns>
    private Vector2 GetCorrectedHomePosition(bool _player2, Vector2 _position, int[] _armSize)
    {
        // 補正値(stub)
        // 補正対象の基準は窓ガラス側のアームのサイズ、補正値の基準は開度100%時に100%降ろしても壁にぶつからない座標
        float[] delta = new float[] { 0, 0, 0.025f, 0.06f };

        // TODO: アームユニットの角度を考慮する
        // TODO: そもそも指定方法を百分率にしたい
        // 候補
        int arm = _armSize[0] > _armSize[1] ? _armSize[1] : _armSize[0];
        float draftXPosition = 0;
        float draftYPosition = -0.3f + _position.y;

        if (!player2)
        {
            // 1P
            draftXPosition = -0.65f + _position.x < -0.65f + _position.x + delta[arm] ?
                -0.65f + _position.x + delta[arm] : -0.65f + _position.x;
        }
        else
        {
            // 2P
            draftXPosition = 0.65f - _position.x > 0.65f - _position.x - delta[arm] ?
                0.65f - _position.x - delta[arm] : 0.65f - _position.x;
        }
        var hoge = new Vector2(0.65f - homePoint.x, -0.3f + homePoint.y);

        return new Vector2(draftXPosition, draftYPosition);
    }

    /// <summary>
    /// Z座標のホームポジションを計算して返す
    /// </summary>
    /// <param name="_homePosition">ホームポジション</param>
    /// <param name="_restriction">位置</param>
    /// <returns>ホームポジション</returns>
    private Vector3 GetZHomePosition(Vector3 _homePosition, float _restriction)
    {
        // TODO: 下降制限位置の移動に依存してるのをやめたい
        _homePosition = new Vector3(_homePosition.x, 1.39f - 0.0068f * _restriction, _homePosition.z);

        return _homePosition;
    }

    /// <summary>
    /// アームの移動速度を設定する
    /// </summary>
    /// <param name="_moveSpeed">移動速度</param>
    private void SetArmMoveSpeed(int _moveSpeed)
    {
        // 移動速度配列
        float[] speedDelta = new float[] {0.002f, 0.0025f, 0.003f, 0.0035f, 0.004f};
        
        // 設定する移動速度
        float setSpeed;
        switch (_moveSpeed)
        {
            case 1:
            case 2:
            case 3:
            case 4:
            case 5:
                setSpeed = speedDelta[_moveSpeed - 1];
                break;
            default:
                setSpeed = speedDelta[2];
                break;
        }

        // 移動速度を設定する
        craneBox.SetMoveSpeed(setSpeed);
    }

    /// <summary>
    /// アームの昇降速度を設定する
    /// </summary>
    /// <param name="_upSpeed">アームの上昇速度</param>
    /// <param name="_downSpeed">アームの降下速度</param>
    private void SetArmLiftSpeed(int _upSpeed, int _downSpeed)
    {
        // 昇降速度配列
        float[] speedDelta = new float[] { 0.002f, 0.0025f, 0.003f, 0.0035f, 0.004f };

        // 設定する上昇速度
        float setUpSpeed;
        switch (_upSpeed)
        {
            case 1:
            case 2:
            case 3:
            case 4:
            case 5:
                setUpSpeed = speedDelta[_upSpeed - 1];
                break;
            default:
                setUpSpeed = speedDelta[2];
                break;
        }

        // 設定する降下速度
        float setDownSpeed;
        switch (_downSpeed)
        {
            case 1:
            case 2:
            case 3:
            case 4:
            case 5:
                setDownSpeed = speedDelta[_downSpeed - 1];
                break;
            default:
                setDownSpeed = speedDelta[2];
                break;
        }

        // 昇降速度を設定する
        lifter.SetUpSpeed(setUpSpeed);
        lifter.SetDownSpeed(setDownSpeed);
    }

    /// <summary>
    /// 7セグ表示の表示モードを変更する
    /// </summary>
    /// <param name="_segMode">表示したいモード</param>
    /// <param name="errorCode">エラーコード(予約)</param>
    private void SetSegDisplay(int _segMode, int _errorCode = -1)
    {
        // 0: クレジット, 1: 残り時間(レバー操作) 8: エラー 9: 休止
        switch (_segMode)
        {
            case 0:
                // 価格情報
                // クレジット2を使わない場合はセグを非表示にする
                preset[0].text = priceSet[0].ToString();
                preset[1].text = priceSet[1] > 0 ? priceSet[1].ToString() : string.Empty;
                preset[2].text = timesSet[0].ToString();
                preset[3].text = priceSet[1] > 0 ? timesSet[1].ToString() : string.Empty;

                // クレジットをダミー投入して残り回数を取得
                int credit = creditSystem.Pay(0);

                // クレジット投入時の表示更新を有効化する
                creditSystem.segUpdateFlag = true;

                // 表示
                // 10クレジットを超えたら9.
                credit3d.text = credit < 10 ? credit.ToString() : "9.";
                limitTimedisplayed.text = credit < 10 ? credit.ToString("D2") : "9.";
                break;

            case 1:
                if (operationType == 0)
                {
                    // ボタン操作の場合は無視
                    break;
                }

                // TODO: 実機確認必要
                // 暫定処理としてクレジット1価格に残り時間を表示、残り回数はそのまま表示、それ以外は消灯
                limitTimedisplayed.text = timer.limitTimeNow.ToString("D2");
                preset[0].text = " " + timer.limitTimeNow.ToString("D2");
                preset[1].text = string.Empty;
                preset[2].text = string.Empty;
                preset[3].text = string.Empty;
                break;

            case 8:
                // エラー
                // TODO: エラーコード表示(実機のエラーコードが判明し次第)
                preset[0].text = SEG_PRICE_ERR;
                preset[1].text = string.Empty;
                preset[2].text = string.Empty;
                preset[3].text = string.Empty;
                credit3d.text = string.Empty;
                break;

            case 9:
                // 休止
                credit3d.text = SEG_REMAIN_NOT;
                preset[0].text = SEG_PRICE_NOT;
                preset[1].text = SEG_PRICE_NOT;
                preset[2].text = SEG_REMAIN_NOT;
                preset[3].text = SEG_REMAIN_NOT;
                break;
        }

    }

    /// <summary>
    /// TYPE-1の効果音定義
    /// </summary>
    private enum TYPE1_SE
    {
        CLENA_FLEX_START,
        CLENA_FLEX_MOVE,
        CLENA_FLEX_DOWN,
        CLENA_FLEX_CATCH,
        CLENA_FLEX_UP,
        CLENA_FLEX_GET,
        CLENA_FLEX_ERROR,
        BIG_CLENA_MOVE,
        BIG_CLENA_DOWN,
        BIG_CLENA_CATCH,
        BIG_CLENA_UP,
    }

    private enum TYPE1_STATUS
    {
        STATUS_INIT_1 = -2,
        STATUS_INIT_2 = -1,
        STATUS_STANDBY = 0,
        STATUS_GAME_START = 1,
        STATUS_X_MOVING = 2,
        STATUS_X_MOVE_END = 3,
        STATUS_Y_MOVING = 4,
        STATUS_Y_MOVE_END = 5,
        STATUS_Z_DESCENDING = 6,
        STATUS_CATCHING = 7,
        STATUS_Z_RIZING = 8,
        STATUS_Z_RIZE_END = 9,
        STATUS_XY_MOVING = 10,
        STATUS_RELEASE = 11,
        STATUS_RELEASE_END = 12,
        STATUS_MOVE_HOME_POSITION = 13,
        STATUS_MOVE_START_POSITION = 14
    }
}