﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

/// <summary>
/// Type1-Hybrid(仮称) Selecterクラス
/// </summary>
public class Type1HSelecter : MonoBehaviour
{
    [SerializeField] Type1HManager[] manager = new Type1HManager[2];
    [SerializeField] Animator[] animator = new Animator[4];
    [SerializeField] int soundType = 0;
    [SerializeField] bool disableBgm = false;
    BGMPlayer bp;

    /// <summary>
    /// 左右どちらかのプレイヤーがプレイ中である
    /// </summary>
    bool isNowPlaying = false;

    /// <summary>
    /// プレイ中のBGM音量(%)
    /// </summary>
    float bgmVolumeInPlaying = 20f;

    /// <summary>
    /// ライトの色
    /// index想定: 0:左 1:右 2:上 3:手元
    /// 0:Disable(未実装) 1:Red, 2:Green, 3:Blue, 4:Yellow, 5:Sky, 6:Purple, 7:Pink(255,128,255), 8:Orange(255,128,0), 9:Forest(128,255,0)
    /// </summary>
    [SerializeField] int[] lightColor = new int[] {0, 0, 0, 0};

    void Start()
    {
        manager[0].soundType = soundType;
        manager[1].soundType = soundType;
        bp = this.transform.Find("BGM").GetComponent<BGMPlayer>();

        for (int i = 0; i < 4; i++)
        {
            // TODO: ライト無効(現状これだとエラーが出る+ライトも消えない)
            //if (lightColor[i] == 0)
            //    break;

            animator[i].SetInteger("ColorNumber", lightColor[i]);
            manager[0].animator[i] = animator[i];
            manager[1].animator[i] = animator[i];
        }
    }

    void Update()
    {
        // BGMを再生しない
        if (disableBgm)
            return;

        if (manager[0].GetStatus() >= 0 && manager[1].GetStatus() >= 0 && !bp.audioSource[soundType].isPlaying)
            bp.Play(soundType);
        // プレイ中は音量を下げる
        if (manager[0].GetStatus() >= 1 || manager[1].GetStatus() >= 1)
        {
            if (!isNowPlaying)
            {
                // プレイ中のBGM音量
                bp.SetAudioVolume((bgmVolumeInPlaying / 100.0f) / 2f);
                isNowPlaying = true;
            }
        }
        else if (manager[0].GetStatus() == 0 && manager[1].GetStatus() == 0)
        {
            if (isNowPlaying)
            {
                // BGMの音量を戻す
                bp.SetAudioVolume(0.5f);
                isNowPlaying = false;
            }
        }
    }

    public Type1HManager GetManager(int num)
    {
        return manager[num - 1];
    }
}
